@@ -24,7 +24,6 @@ import (
 	"context"
 	"errors"
 	"fmt"
-	"slices"
 	"sync"
 	"time"
 
@@ -100,7 +99,6 @@ func NewDesiredStateOfWorldPopulator(
 	desiredStateOfWorld cache.DesiredStateOfWorld,
 	actualStateOfWorld cache.ActualStateOfWorld,
 	kubeContainerRuntime kubecontainer.Runtime,
-	keepTerminatedPodVolumes bool,
 	csiMigratedPluginManager csimigration.PluginManager,
 	intreeToCSITranslator csimigration.InTreeToCSITranslator,
 	volumePluginMgr *volume.VolumePluginMgr) DesiredStateOfWorldPopulator {
@@ -114,7 +112,6 @@ func NewDesiredStateOfWorldPopulator(
 		pods: processedPods{
 			processedPods: make(map[volumetypes.UniquePodName]bool)},
 		kubeContainerRuntime:     kubeContainerRuntime,
-		keepTerminatedPodVolumes: keepTerminatedPodVolumes,
 		hasAddedPods:             false,
 		hasAddedPodsLock:         sync.RWMutex{},
 		csiMigratedPluginManager: csiMigratedPluginManager,
@@ -132,7 +129,6 @@ type desiredStateOfWorldPopulator struct {
 	actualStateOfWorld       cache.ActualStateOfWorld
 	pods                     processedPods
 	kubeContainerRuntime     kubecontainer.Runtime
-	keepTerminatedPodVolumes bool
 	hasAddedPods             bool
 	hasAddedPodsLock         sync.RWMutex
 	csiMigratedPluginManager csimigration.PluginManager
@@ -235,9 +231,6 @@ func (dswp *desiredStateOfWorldPopulator) findAndRemoveDeletedPods() {
 			if !dswp.podStateProvider.ShouldPodRuntimeBeRemoved(pod.UID) {
 				continue
 			}
-			if dswp.keepTerminatedPodVolumes {
-				continue
-			}
 		}
 
 		// Once a pod has been deleted from kubelet pod manager, do not delete
@@ -462,7 +455,7 @@ func (dswp *desiredStateOfWorldPopulator) deleteProcessedPod(
 // specified volume. It dereference any PVC to get PV objects, if needed.
 // Returns an error if unable to obtain the volume at this time.
 func (dswp *desiredStateOfWorldPopulator) createVolumeSpec(
-	podVolume v1.Volume, pod *v1.Pod, mounts, devices sets.String) (*v1.PersistentVolumeClaim, *volume.Spec, string, error) {
+	podVolume v1.Volume, pod *v1.Pod, mounts, devices sets.Set[string]) (*v1.PersistentVolumeClaim, *volume.Spec, string, error) {
 	pvcSource := podVolume.VolumeSource.PersistentVolumeClaim
 	isEphemeral := pvcSource == nil && podVolume.VolumeSource.Ephemeral != nil
 	if isEphemeral {
@@ -565,21 +558,15 @@ func (dswp *desiredStateOfWorldPopulator) getPVCExtractPV(
 		return nil, fmt.Errorf("failed to fetch PVC from API server: %v", err)
 	}
 
-	// Pods that uses a PVC that is being deleted and not protected by
-	// kubernetes.io/pvc-protection must not be started.
+	// Pods that uses a PVC that is being deleted must not be started.
 	//
-	// 1) In case an old kubelet is running without this check, the worst
-	// that can happen is that such pod is scheduled. This was the default
-	// behavior in 1.8 and earlier and users should not be that surprised.
+	// In case an old kubelet is running without this check or some kubelets
+	// have this feature disabled, the worst that can happen is that such
+	// pod is scheduled. This was the default behavior in 1.8 and earlier
+	// and users should not be that surprised.
 	// It should happen only in very rare case when scheduler schedules
 	// a pod and user deletes a PVC that's used by it at the same time.
-	//
-	// 2) Adding a check for kubernetes.io/pvc-protection here to prevent
-	// the existing running pods from being affected during the rebuild of
-	// the desired state of the world cache when the kubelet is restarted.
-	// It is safe for kubelet to add this check here because the PVC will
-	// be stuck in Terminating state until the pod is deleted.
-	if pvc.ObjectMeta.DeletionTimestamp != nil && !slices.Contains(pvc.Finalizers, util.PVCProtectionFinalizer) {
+	if pvc.ObjectMeta.DeletionTimestamp != nil {
 		return nil, errors.New("PVC is being deleted")
 	}
 
